#include <stdint.h>

// CS pin at PD13
sbit Chip_Select_Direction at  TRISE0_bit;
sbit Chip_Select at RE0_bit;


typedef struct MLX
{
      unsigned int angle;
      unsigned int error_code;
      unsigned char buffer[4];
} mlx_t;

mlx_t mlx;

int read_mlx(){
   uint16_t datavalue = 0;
   uint16_t inverted_datavalue = 0;
   uint16_t checkvalue = 0;

  Chip_Select = 0;
  delay_ms(4);
  SPI1_Write(0xAA);
  SPI1_Write(0xFF);
  delay_uS(50);
  mlx.buffer[3]=SPI1_Read(0xFF);
  mlx.buffer[2]=SPI1_Read(0xFF);
  delay_uS(40);
  mlx.buffer[1]=SPI1_Read(0xFF);
  mlx.buffer[0]=SPI1_Read(0xFF);
  delay_us(10);

  Chip_Select = 1;


  datavalue = ((uint16_t) mlx.buffer[3]) << 8;
  datavalue |= ((uint16_t) mlx.buffer[2]);

  checkvalue = datavalue & 1;   // LSB signifies an error, check if there is an error in measurement

#if defined( __MIKROC_PRO_FOR_PIC__ )
  if (checkvalue == 0)
     return -1;
#else
  if (checkvalue == 1)
     return -1;
#endif

  inverted_datavalue = ((uint16_t) mlx.buffer[1]) << 8;
  inverted_datavalue |= ((uint16_t) mlx.buffer[0]);

  // two data bytes are followed by two inverted data bytes, if those allign, then measurement is correct
  if (datavalue != ~inverted_datavalue)
     return -1;

  else
  {
      mlx.angle = datavalue >> 2;
      return 0;
  }

}


void main() {

    char txt[20];
    uint8_t checkit= 0;
    float angle = 0.0;

    UART1_Init( 57600 );
    Delay_ms(300);

    UART1_Write_Text("Uart initialized\r\n");
    Chip_Select_Direction = 0;

    // Initialize SPI
    SPI1_Init_Advanced( _SPI_MASTER_OSC_DIV64,  _SPI_DATA_SAMPLE_END, _SPI_CLK_IDLE_LOW, _SPI_HIGH_2_LOW );
    Delay_ms(400);


    while (1)
    {
        checkit = read_mlx();
        if ( checkit == 0)                          // if read was successful, print success and angle
        {
            Uart1_Write_Text("Success \r\n");
            angle = (float) mlx.angle;
            angle /= 45.5;
            floattostr(angle,txt);
            Uart1_Write_Text(txt);
            Uart1_Write_Text("\r\n");
         }
         else                                     // else print fail
             Uart1_Write_Text("Fail \r\n");

      Delay_ms(400);                // Wait 400ms
    }
}